#!/usr/bin/python
#-*- coding: utf-8 -*-

"""
SAT communication bridge
Copyright (C) 2009, 2010, 2011  Jérôme Poisson (goffi@goffi.org)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from bridge_frontend import BridgeFrontend
import dbus, dbus.glib
from logging import debug

class BridgeExceptionNoService(Exception):
    pass

class DBusBridgeFrontend(BridgeFrontend):
    def __init__(self):
        try:
            self.sessions_bus = dbus.SessionBus()
            self.db_object = self.sessions_bus.get_object('org.goffi.SAT',
                                '/org/goffi/SAT/bridge')
            self.db_comm_iface = dbus.Interface(self.db_object,
                                dbus_interface='org.goffi.SAT.communication')
            self.db_req_iface = dbus.Interface(self.db_object,
                                dbus_interface='org.goffi.SAT.request')
        except dbus.exceptions.DBusException,e:
            if e._dbus_error_name=='org.freedesktop.DBus.Error.ServiceUnknown':
                raise BridgeExceptionNoService
            else:
                raise e
        #props = self.db_comm_iface.getProperties()

    def register(self, functionName, handler, iface="communication"):
        if iface == "communication":
            self.db_comm_iface.connect_to_signal(functionName, handler)
        elif iface == "request":
            self.db_req_iface.connect_to_signal(functionName, handler)

    def addContact(self, entity, profile_key="@DEFAULT@"):
        debug ("addContact")
        return self.db_comm_iface.addContact(entity, profile_key)

    def callMenu(self, category, name, menu_type, profile_key):
        debug ("callMenu")
        return unicode(self.db_req_iface.callMenu(category, name, menu_type, profile_key))

    def confirmationAnswer(self, id, accepted, data):
        debug ("confirmationAnswer")
        return self.db_req_iface.confirmationAnswer(id, accepted, data)

    def connect(self, profile_key="@DEFAULT@"):
        debug ("connect")
        return self.db_comm_iface.connect(profile_key)

    def createProfile(self, profile):
        debug ("createProfile")
        return self.db_req_iface.createProfile(profile)

    def delContact(self, entity, profile_key="@DEFAULT@"):
        debug ("delContact")
        return self.db_comm_iface.delContact(entity, profile_key)

    def deleteProfile(self, profile):
        debug ("deleteProfile")
        return self.db_req_iface.deleteProfile(profile)

    def disconnect(self, profile_key="@DEFAULT@"):
        debug ("disconnect")
        return self.db_comm_iface.disconnect(profile_key)

    def getContacts(self, profile_key="@DEFAULT@"):
        debug ("getContacts")
        return self.db_comm_iface.getContacts(profile_key)

    def getHistory(self, from_jid, to_jid, size):
        debug ("getHistory")
        return self.db_comm_iface.getHistory(from_jid, to_jid, size)

    def getMenuHelp(self, category, name, menu_type):
        debug ("getMenuHelp")
        return unicode(self.db_req_iface.getMenuHelp(category, name, menu_type))

    def getMenus(self, ):
        debug ("getMenus")
        return self.db_req_iface.getMenus()

    def getParamA(self, name, category, attribute="value", profile_key="@DEFAULT@"):
        debug ("getParamA")
        return unicode(self.db_comm_iface.getParamA(name, category, attribute, profile_key))

    def getParams(self, profile_key="@DEFAULT@"):
        debug ("getParams")
        return unicode(self.db_comm_iface.getParams(profile_key))

    def getParamsCategories(self, ):
        debug ("getParamsCategories")
        return self.db_comm_iface.getParamsCategories()

    def getParamsForCategory(self, category, profile_key="@DEFAULT@"):
        debug ("getParamsForCategory")
        return unicode(self.db_comm_iface.getParamsForCategory(category, profile_key))

    def getParamsUI(self, profile_key="@DEFAULT@"):
        debug ("getParamsUI")
        return unicode(self.db_comm_iface.getParamsUI(profile_key))

    def getPresenceStatus(self, profile_key="@DEFAULT@"):
        debug ("getPresenceStatus")
        return self.db_comm_iface.getPresenceStatus(profile_key)

    def getProfileName(self, profile_key="@DEFAULT@"):
        debug ("getProfileName")
        return unicode(self.db_req_iface.getProfileName(profile_key))

    def getProfilesList(self, ):
        debug ("getProfilesList")
        return self.db_req_iface.getProfilesList()

    def getProgress(self, id):
        debug ("getProgress")
        return self.db_req_iface.getProgress(id)

    def getVersion(self, ):
        debug ("getVersion")
        return unicode(self.db_req_iface.getVersion())

    def getWaitingSub(self, profile_key="@DEFAULT@"):
        debug ("getWaitingSub")
        return self.db_comm_iface.getWaitingSub(profile_key)

    def isConnected(self, profile_key="@DEFAULT@"):
        debug ("isConnected")
        return self.db_comm_iface.isConnected(profile_key)

    def launchAction(self, action_type, data, profile_key="@DEFAULT@"):
        debug ("launchAction")
        return unicode(self.db_req_iface.launchAction(action_type, data, profile_key))

    def registerNewAccount(self, login, password, host, port=5222):
        debug ("registerNewAccount")
        return unicode(self.db_comm_iface.registerNewAccount(login, password, host, port))

    def sendMessage(self, to_jid, message, subject='', mess_type="chat", profile_key="@DEFAULT@"):
        debug ("sendMessage")
        return self.db_comm_iface.sendMessage(to_jid, message, subject, mess_type, profile_key)

    def setParam(self, name, value, category, profile_key="@DEFAULT@"):
        debug ("setParam")
        return self.db_comm_iface.setParam(name, value, category, profile_key)

    def setPresence(self, to_jid='', show='', priority=0, statuses={}, profile_key="@DEFAULT@"):
        debug ("setPresence")
        return self.db_comm_iface.setPresence(to_jid, show, priority, statuses, profile_key)

    def subscription(self, sub_type, entity, profile_key="@DEFAULT@"):
        debug ("subscription")
        return self.db_comm_iface.subscription(sub_type, entity, profile_key)


#methods from plugins
    def getRoomJoined(self, profile_key='@DEFAULT@'):
        return self.db_comm_iface.getRoomJoined(profile_key)

    def getRoomSubjects(self, profile_key='@DEFAULT@'):
        return self.db_comm_iface.getRoomSubjects(profile_key)

    def joinMUC(self, service, roomId, nick, profile_key='@DEFAULT@'):
        return self.db_comm_iface.joinMUC(service, roomId, nick, profile_key)

    def tarotGameCreate(self, room_jid, players, profile_key='@DEFAULT@'):
        return self.db_comm_iface.tarotGameCreate(room_jid, players, profile_key)

    def tarotGameReady(self, player, referee, profile_key='@DEFAULT@'):
        return self.db_comm_iface.tarotGameReady(player, referee, profile_key)

    def tarotGameContratChoosed(self, player, referee, contrat, profile_key='@DEFAULT@'):
        return self.db_comm_iface.tarotGameContratChoosed(player, referee, contrat, profile_key)

    def tarotGamePlayCards(self, player, referee, cards, profile_key='@DEFAULT@'):
        return self.db_comm_iface.tarotGamePlayCards(player, referee, cards, profile_key)

    def sendFile(self, to, path, profile_key='@DEFAULT@'):
        return self.db_comm_iface.sendFile(to, path, profile_key)

    def findGateways(self, target, profile_key='@DEFAULT@'):
        return self.db_comm_iface.findGateways(target, profile_key)

    def getCard(self, target, profile_key='@DEFAULT@'):
        return self.db_comm_iface.getCard(target, profile_key)

    def getCardCache(self, target):
        return self.db_comm_iface.getCardCache(target)

    def getAvatarFile(self, hash):
        return self.db_comm_iface.getAvatarFile(hash)

    def in_band_register(self, target, profile_key='@DEFAULT@'):
        return self.db_comm_iface.in_band_register(target, profile_key)

    def gatewayRegister(self, action, target, data, profile_key='@DEFAULT@'):
        if data == None:
            data = [('', '')] #XXX: we have to do this awful hack because python dbus need to guess the signature
        return self.db_req_iface.gatewayRegister(action, target, data, profile_key)

